"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ALGOImplementation = void 0;
const msgpack = __importStar(require("algo-msgpack-with-bigint"));
const axios_1 = __importDefault(require("axios"));
const __1 = require("..");
const axios_rate_limit_1 = __importDefault(require("../axios-rate-limiter/axios-rate-limit"));
const StatusBase_1 = require("../base-objects/StatusBase");
const types_1 = require("../types");
const algoUtils_1 = require("../utils/algoUtils");
const errors_1 = require("../utils/errors");
const utils_1 = require("../utils/utils");
const DEFAULT_TIMEOUT = 60000;
const DEFAULT_RATE_LIMIT_OPTIONS = {
    maxRPS: 5,
};
function algoResponseValidator(responseCode) {
    // allow any response, process them later in mcc
    return responseCode >= 200 && responseCode < 600;
}
class ALGOImplementation {
    constructor(createConfig) {
        this.createConfig = createConfig;
        const algodClient = axios_1.default.create({
            baseURL: createConfig.algod.url,
            timeout: DEFAULT_TIMEOUT,
            headers: {
                "Content-Type": "application/json",
                "X-Algo-API-Token": createConfig.algod.token,
            },
            validateStatus: algoResponseValidator,
        });
        this.algodClient = (0, axios_rate_limit_1.default)(algodClient, Object.assign(Object.assign({}, DEFAULT_RATE_LIMIT_OPTIONS), createConfig.rateLimitOptions));
        if (createConfig.indexer) {
            const indexerClient = axios_1.default.create({
                baseURL: createConfig.indexer.url,
                timeout: DEFAULT_TIMEOUT,
                headers: {
                    "Content-Type": "application/json",
                    "X-Algo-API-Token": createConfig.indexer.token,
                },
                validateStatus: algoResponseValidator,
            });
            this.indexerClient = (0, axios_rate_limit_1.default)(indexerClient, Object.assign(Object.assign({}, DEFAULT_RATE_LIMIT_OPTIONS), createConfig.rateLimitOptions));
        }
        if (!createConfig.inRegTest) {
            this.inRegTest = false;
        }
        else {
            this.inRegTest = createConfig.inRegTest;
        }
        this.chainType = types_1.ChainType.ALGO;
    }
    /**
     * Return node status object for Algo node
     */
    getNodeStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get("health");
            (0, algoUtils_1.algo_ensure_data)(res);
            let ver = yield this.algodClient.get("versions");
            (0, algoUtils_1.algo_ensure_data)(ver);
            let status = yield this.algodClient.get("/v2/status");
            (0, algoUtils_1.algo_ensure_data)(status);
            status = (0, utils_1.toCamelCase)(status.data);
            const statusData = {
                health: res.status,
                status: status,
                versions: (0, utils_1.toCamelCase)(ver.data),
            };
            return new StatusBase_1.AlgoNodeStatus(statusData);
        });
    }
    getBottomBlockHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            let bottomBlockHeight = yield this.getBlockHeight();
            // check -1.000 -10.000 and -100.000 blocs
            for (let checkRound = 0; checkRound < 3; checkRound++) {
                bottomBlockHeight -= Math.pow(10, 3 + checkRound);
                let blc = yield this.algodClient.get(`/v2/blocks/${bottomBlockHeight}`);
                if (blc.status !== 200) {
                    // we didn't get block
                    for (let i = 0; i < 10; i++) {
                        bottomBlockHeight += Math.pow(10, 2 + checkRound);
                        blc = yield this.algodClient.get(`/v2/blocks/${bottomBlockHeight}`);
                        if (blc.status === 200) {
                            break;
                        }
                    }
                    // If we ever come here we are not healthy
                }
            }
            return bottomBlockHeight;
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Block methods //////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    getBlock(round) {
        return __awaiter(this, void 0, void 0, function* () {
            if (round === undefined) {
                const status = yield this.getStatus();
                round = status.lastRound;
            }
            let res = yield this.algodClient.get(`/v2/blocks/${round}?format=msgpack`, {
                responseType: "arraybuffer",
                headres: { "Content-Type": "application/msgpack" },
            });
            if ((0, algoUtils_1.algo_check_expect_block_out_of_range)(res)) {
                throw new errors_1.mccError(errors_1.mccErrorCode.InvalidBlock);
            }
            (0, algoUtils_1.algo_ensure_data)(res);
            const decoded = (0, algoUtils_1.mpDecode)(res.data);
            return new __1.AlgoBlock(decoded);
        });
    }
    getBlockHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            const blockData = yield this.getBlockHeader();
            return blockData.block.rnd;
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Transaction methods ////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    /**
     * get Transaction cannot be called on algorand's algod client, use getIndexerTransaction instead or get transactions from block
     * @param txid
     */
    getTransaction(txid) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new errors_1.mccError(errors_1.mccErrorCode.InvalidMethodCall);
        });
    }
    listTransactions(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.createConfig.indexer) {
                // No indexer
                return null;
            }
            let snakeObject = {};
            if (options !== undefined) {
                snakeObject = (0, utils_1.toSnakeCase)(options);
            }
            let res = yield this.indexerClient.get(`/v2/transactions`, {
                params: snakeObject,
            });
            (0, algoUtils_1.algo_ensure_data)(res);
            let camelList = {
                currentRound: res.data["current-round"],
                nextToken: res.data["next-token"],
                transactions: [],
            };
            for (let key of Object.keys(res.data.transactions)) {
                camelList.transactions.push((0, utils_1.toCamelCase)(res.data.transactions[key]));
            }
            return camelList;
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Client helper (private) methods ////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    getStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get("v2/status");
            (0, algoUtils_1.algo_ensure_data)(res);
            return (0, utils_1.toCamelCase)(res.data);
        });
    }
    getBlockHeaderCert(round) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get(`/v2/blocks/${round}?format=msgpack`, {
                responseType: "arraybuffer",
                headres: { "Content-Type": "application/msgpack" },
            });
            (0, algoUtils_1.algo_ensure_data)(res);
            const decoded = msgpack.decode(res.data);
            return decoded.cert;
        });
    }
    getBlockHeader(round) {
        return __awaiter(this, void 0, void 0, function* () {
            if (round === undefined) {
                const status = yield this.getStatus();
                round = status.lastRound;
            }
            let resorig = yield this.algodClient.get(`/v2/blocks/${round}`);
            (0, algoUtils_1.algo_ensure_data)(resorig);
            let responseData = (0, utils_1.toCamelCase)(resorig.data);
            responseData.type = "IAlgoGetBlockHeaderRes";
            return responseData;
        });
    }
}
exports.ALGOImplementation = ALGOImplementation;
//# sourceMappingURL=AlgoRpcImplementation.js.map