"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlgoBlock = void 0;
const globalSettings_1 = require("../../global-settings/globalSettings");
const algoUtils_1 = require("../../utils/algoUtils");
const managed_1 = require("../../utils/managed");
const utils_1 = require("../../utils/utils");
const BlockBase_1 = require("../BlockBase");
const TransactionBase_1 = require("../TransactionBase");
let AlgoBlock = class AlgoBlock extends BlockBase_1.BlockBase {
    constructor(data) {
        super(data);
        this.transactionObjects = [];
        this.processTransactions();
    }
    get number() {
        var _a, _b;
        return (_b = (_a = this.data) === null || _a === void 0 ? void 0 : _a.block) === null || _b === void 0 ? void 0 : _b.rnd;
    }
    get blockHash() {
        return (0, algoUtils_1.hexToBase64)(this.data.cert.prop.dig);
    }
    // Algo special
    get blockHashBase32() {
        return (0, algoUtils_1.hexToBase32)(this.data.cert.prop.dig);
    }
    // Algo special
    get blockHashBase64() {
        return (0, algoUtils_1.hexToBase64)(this.data.cert.prop.dig);
    }
    get stdBlockHash() {
        var _a, _b, _c;
        return (0, algoUtils_1.bytesToHex)((_c = (_b = (_a = this.data) === null || _a === void 0 ? void 0 : _a.cert) === null || _b === void 0 ? void 0 : _b.prop) === null || _c === void 0 ? void 0 : _c.dig);
    }
    get unixTimestamp() {
        var _a, _b;
        return (_b = (_a = this.data) === null || _a === void 0 ? void 0 : _a.block) === null || _b === void 0 ? void 0 : _b.ts;
    }
    get transactionIds() {
        return this.transactionObjects.map((trasn) => {
            return trasn.txid;
        });
    }
    get stdTransactionIds() {
        return this.transactionObjects.map((trasn) => {
            return trasn.stdTxid;
        });
    }
    get transactions() {
        return this.transactionObjects;
    }
    get transactionCount() {
        var _a, _b, _c;
        if (!this.data.block.txns) {
            return 0;
        }
        return (_c = (_b = (_a = this.data) === null || _a === void 0 ? void 0 : _a.block) === null || _b === void 0 ? void 0 : _b.txns) === null || _c === void 0 ? void 0 : _c.length;
    }
    ////////////////////////////////////////
    //// Additional transaction objects ////
    ////////////////////////////////////////
    processTransactions() {
        this.transactionObjects = [];
        for (let transactionBase of this.data.block.txns) {
            try {
                // we have to ensure all addresses are buffers with checksum
                if (transactionBase.txn.rcv)
                    transactionBase.txn.rcv = (0, algoUtils_1.bufAddToCBufAdd)(transactionBase.txn.rcv);
                if (transactionBase.txn.snd)
                    transactionBase.txn.snd = (0, algoUtils_1.bufAddToCBufAdd)(transactionBase.txn.snd);
                if (transactionBase.txn.arcv)
                    transactionBase.txn.arcv = (0, algoUtils_1.bufAddToCBufAdd)(transactionBase.txn.arcv);
                const st = new algoUtils_1.SignedTransactionWithAD(this.data.block.gh, this.data.block.gen, transactionBase);
                const data = Object.assign({ txid: st.txn.txn.txID(), timestamp: this.unixTimestamp, hgi: transactionBase.hgi }, transactionBase.txn);
                if (transactionBase.sig)
                    data.sig = transactionBase.sig;
                if (transactionBase.lsig)
                    data.lsig = transactionBase.lsig;
                if (transactionBase.msig)
                    data.msig = transactionBase.msig;
                if (transactionBase.sgnr)
                    data.sgnr = transactionBase.sgnr;
                this.transactionObjects.push(new TransactionBase_1.AlgoTransaction(data));
            }
            catch (e) {
                // TODO logger
                // TODO What happens if there is a transaction we dont know how to process
                globalSettings_1.mccSettings.loggingCallback(`Unable to process transaction ${(0, utils_1.mccJsonStringify)(transactionBase)}`);
                // throw e;
            }
        }
    }
};
AlgoBlock = __decorate([
    (0, managed_1.Managed)()
], AlgoBlock);
exports.AlgoBlock = AlgoBlock;
//# sourceMappingURL=AlgoBlock.js.map