"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlgoTransaction = void 0;
const types_1 = require("../../types");
const algoUtils_1 = require("../../utils/algoUtils");
const constants_1 = require("../../utils/constants");
const managed_1 = require("../../utils/managed");
const utils_1 = require("../../utils/utils");
const TransactionBase_1 = require("../TransactionBase");
const web3 = require("web3");
/**
 * docs https://developer.algorand.org/docs/get-details/transactions/transactions/
 */
let AlgoTransaction = class AlgoTransaction extends TransactionBase_1.TransactionBase {
    get txid() {
        return this.data.txid;
    }
    get stdTxid() {
        return (0, algoUtils_1.base32ToHex)(this.data.txid);
    }
    get hash() {
        return this.txid;
    }
    get reference() {
        if (this.data.note) {
            return [(0, algoUtils_1.bytesToHex)(this.data.note)];
        }
        return [""];
    }
    get stdPaymentReference() {
        var _a;
        let paymentReference = ((_a = this.reference) === null || _a === void 0 ? void 0 : _a.length) === 1 ? (0, utils_1.prefix0x)(this.reference[0]) : "";
        try {
            // try to parse out
            paymentReference = (0, utils_1.prefix0x)(web3.utils.hexToString((0, utils_1.prefix0x)(paymentReference)));
        }
        catch (e) {
            return utils_1.ZERO_BYTES_32;
        }
        if (!(0, utils_1.isValidBytes32Hex)(paymentReference)) {
            paymentReference = utils_1.ZERO_BYTES_32;
        }
        return paymentReference;
    }
    get unixTimestamp() {
        return this.data.timestamp;
    }
    get sourceAddresses() {
        return [(0, algoUtils_1.hexToBase32)(this.data.snd)];
    }
    get receivingAddresses() {
        // for transactions of type pay
        if (this.data.rcv) {
            return [(0, algoUtils_1.hexToBase32)(this.data.rcv)];
        }
        // for transactions of type axfer
        else if (this.data.arcv) {
            return [(0, algoUtils_1.hexToBase32)(this.data.arcv)];
        }
        return [];
    }
    get fee() {
        return (0, utils_1.toBN)(this.data.fee || 0);
    }
    get spentAmounts() {
        // for transactions of type pay
        if (this.data.amt) {
            let amount = this.data.amt.toString();
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee.add((0, utils_1.toBN)(amount)),
                },
            ];
        }
        // for transactions of type axfer
        // TODO
        if (this.data.aamt) {
            let amount = this.data.aamt.toString();
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee,
                },
            ];
        }
        return [
            {
                address: this.sourceAddresses[0],
                amount: this.fee,
            },
        ];
    }
    get receivedAmounts() {
        // for transactions of type pay
        if (this.data.amt) {
            let amount = this.data.amt.toString();
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(amount),
                },
            ];
        }
        // for transactions of type axfer
        if (this.data.aamt) {
            let amount = this.data.aamt.toString();
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(amount),
                },
            ];
        }
        return [];
    }
    get type() {
        return this.data.type;
    }
    get isNativePayment() {
        return this.type === "pay";
    }
    get currencyName() {
        if (this.type === "pay") {
            return constants_1.ALGO_NATIVE_TOKEN_NAME;
        }
        else if (this.type === "axfer" && this.data.xaid) {
            return this.data.xaid.toString();
        }
        return "";
    }
    get elementaryUnits() {
        return (0, utils_1.toBN)(constants_1.ALGO_MDU);
    }
    get successStatus() {
        // TODO research this further
        return types_1.TransactionSuccessStatus.SUCCESS;
    }
    paymentSummary(client, inUtxo, utxo, makeFullPayment) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isNativePayment) {
                return { isNativePayment: false };
            }
            return {
                isNativePayment: true,
                sourceAddress: this.sourceAddresses[0],
                receivingAddress: this.receivingAddresses[0],
                spentAmount: this.spentAmounts[0].amount,
                receivedAmount: this.receivedAmounts[0].amount,
                paymentReference: this.stdPaymentReference,
                oneToOne: true,
                isFull: true,
            };
        });
    }
};
AlgoTransaction = __decorate([
    (0, managed_1.Managed)()
], AlgoTransaction);
exports.AlgoTransaction = AlgoTransaction;
//# sourceMappingURL=AlgoTransaction.js.map