export declare enum TraceMethodType {
    Descriptor = 0,
    Getter = 1,
    Setter = 2,
    Function = 3
}
export declare class TraceMethod {
    className: string;
    methodName: string;
    methodType: TraceMethodType;
    source: string;
    calls: number;
    inclusiveTime: number;
    innerTime: number;
    methodsCalled: TraceMethod[];
    constructor(className: string, methodName: string, methodType: TraceMethodType);
}
export declare class TraceStack {
    method: TraceMethod;
    args: any[];
    ret: any;
    callIndex: number;
    completed: boolean;
    startTime: number;
    endTime: number;
    error: Error | undefined;
    stack: string;
    level: number;
    constructor(method: TraceMethod, args: any[]);
    get systemTime(): number;
    get traceTime(): number;
    start(): void;
    complete(ret: any): void;
    completeException(error: Error): void;
    get argsToString(): string;
    toString(showIndent?: boolean, showSource?: boolean, showTiming?: boolean): string;
    toShortString(): string;
}
export declare class TraceManager {
    displayTrace: boolean;
    methods: TraceMethod[];
    stack: TraceStack[];
    trace: TraceStack[];
    constructor();
    createMethod(className: string, methodName: string, methodType: TraceMethodType): TraceMethod;
    start(className: string, methodName: string, args: any[], methodType: TraceMethodType): TraceStack;
    complete(ret: any): void;
    completeError(error: Error): void;
    displayError(error: any): void;
    showStack(): void;
    showTrace(indent?: boolean, source?: boolean, timing?: boolean): void;
    showMethods(indent?: boolean, source?: boolean, timing?: boolean): void;
    clearTrace(): void;
    get firstTrace(): string;
    get lastTrace(): string;
}
export declare const traceManager: TraceManager;
export declare function isPromise(p: any): boolean;
export declare function RegisterTraceValue(target: any, name?: string, descriptor?: any): any;
export declare function RegisterTraceGetter(target: any, name?: string, descriptor?: any): any;
export declare function RegisterTraceSetter(target: any, name?: string, descriptor?: any): any;
export declare function RegisterTraceClass(targetClass: any): any;
export declare function round(x: number, decimal?: number): number;
//# sourceMappingURL=trace.d.ts.map