"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const BtcMccConnection = {
    url: process.env.BTC_URL || "",
    username: process.env.BTC_USERNAME || "",
    password: process.env.BTC_PASSWORD || "",
};
describe("BTC mainnet client tests", () => {
    let MccClient;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.BTC(BtcMccConnection);
        });
    });
    describe("Basic functionality ", function () {
        it("Should say it is BTC mcc client ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(MccClient.chainType).to.eq(src_1.ChainType.BTC);
            });
        });
        it("Should get block header ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const blockhash = "00000000000000000009a57dd6ca3b8d18af776732364dfc17b07b5bf67c5368";
                let header = yield MccClient.getBlockHeader(blockhash);
                // console.log(header);
                (0, chai_1.expect)(header.hash).to.eq(blockhash);
            });
        });
        it("Should get null for wrong header ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const blockhash = "00000000000000000009a57dd6ca3b8d18af776732364dfc17b07b5bf67c53ff";
                let header = yield MccClient.getBlockHeader(blockhash);
                // console.log(header);
                (0, chai_1.expect)(header).to.equal(null);
            });
        });
        it("Should get null from get block fow invalid data ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const blockhash = 1000000;
                let header = yield MccClient.getBlock(blockhash);
                // console.log(header);
                (0, chai_1.expect)(header).to.equal(null);
            });
        });
    });
    describe("OP_RETURN script", function () {
        it("should get OP_RETURN message from transaction ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                const txid = "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d3778615684";
                let trans = yield BtcRpc.getTransaction(txid);
                if (trans) {
                    let reference = trans.reference;
                    (0, chai_1.expect)(reference.length).to.eq(1);
                    (0, chai_1.expect)(reference[0]).to.eq("636861726c6579206c6f766573206865696469");
                }
                else {
                    (0, chai_1.expect)(trans).to.eq(null);
                }
            });
        });
        it("should get OP_RETURN message from transaction ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                const txid = "4dd738c1aebb7a1e50f36100779369053fffd790acfacd0ab10dbbb587e4af01";
                let trans = yield BtcRpc.getTransaction(txid);
                if (trans) {
                    let reference = trans.reference;
                    (0, chai_1.expect)(reference.length).to.eq(1);
                }
                else {
                    (0, chai_1.expect)(trans).to.eq(null);
                }
            });
        });
        it("should get multiple OP_RETURN message from transaction ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                const coinbaseTx = "fbf351cd9f1a561be21e3977282e931c5209ed6b90472e225b4a674dbc643511";
                let trans = yield BtcRpc.getTransaction(coinbaseTx);
                if (trans) {
                    let reference = trans.reference;
                    (0, chai_1.expect)(reference.length).to.eq(3);
                }
                else {
                    (0, chai_1.expect)(trans).to.eq(null);
                }
            });
        });
    });
    describe("ChainTips", function () {
        return __awaiter(this, void 0, void 0, function* () {
            it("basic chaintips ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks();
                    console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("full chaintips ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks({ all_blocks: true });
                    console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("chaintips after ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks({ height_gte: 706000 });
                    console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("chaintips after ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks({ height_gte: 706000, all_blocks: true });
                    console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("All Block tips ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(BtcMccConnection);
                    const BlockTips = yield BtcRpc.getTopLiteBlocks(6);
                    console.log(BlockTips);
                    // console.log(BlockTips.length)
                });
            });
        });
    });
});
//# sourceMappingURL=mainNetBtc.test.js.map