"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.utxo_ensure_data = exports.utxo_check_expect_block_out_of_range = exports.utxo_check_expect_empty = exports.recursive_block_tip = exports.recursive_block_hash = exports.WordToOpcode = void 0;
const LiteBlock_1 = require("../base-objects/blocks/LiteBlock");
const utils_1 = require("./utils");
/**
 * Opcode words to their opcodes
 * @dev https://en.bitcoin.it/wiki/Script
 */
var WordToOpcode;
(function (WordToOpcode) {
    WordToOpcode[WordToOpcode["OP_RETURN"] = 106] = "OP_RETURN";
})(WordToOpcode = exports.WordToOpcode || (exports.WordToOpcode = {}));
function recursive_block_hash(clinet, hash, processHeight) {
    return __awaiter(this, void 0, void 0, function* () {
        if (hash === "") {
            return [];
        }
        if (processHeight <= 1) {
            return [hash];
        }
        else {
            const Cblock = yield clinet.getBlockHeader(hash);
            const hs = (Cblock === null || Cblock === void 0 ? void 0 : Cblock.previousblockhash) || "";
            return (yield recursive_block_hash(clinet, hs, processHeight - 1)).concat([hash]);
        }
    });
}
exports.recursive_block_hash = recursive_block_hash;
function recursive_block_tip(clinet, tip, processHeight) {
    return __awaiter(this, void 0, void 0, function* () {
        if (tip.stdBlockHash === "") {
            return [];
        }
        const tempTip = new LiteBlock_1.LiteBlock({ hash: tip.stdBlockHash, number: tip.number });
        if (processHeight <= 1) {
            return [tempTip];
        }
        else {
            const CurrBlock = yield clinet.getBlockHeader(tip.stdBlockHash);
            const previousHash = (CurrBlock === null || CurrBlock === void 0 ? void 0 : CurrBlock.previousblockhash) || "";
            const previousHeight = (CurrBlock === null || CurrBlock === void 0 ? void 0 : CurrBlock.height) - 1 || 0;
            return (yield recursive_block_tip(clinet, new LiteBlock_1.LiteBlock({ hash: previousHash, number: previousHeight }), processHeight - 1)).concat([tempTip]);
        }
    });
}
exports.recursive_block_tip = recursive_block_tip;
////////////////////////////
//// MCC Error handling ////
////////////////////////////
/**
 * Check if return error code is -5 (can't find transaction)
 * @param data
 * @returns
 */
function utxo_check_expect_empty(data) {
    if (!data || !data.error) {
        return false;
    }
    if (data.error.code && data.error.code === -5) {
        return true;
    }
    else {
        return false;
        // throw MccError(data);
    }
}
exports.utxo_check_expect_empty = utxo_check_expect_empty;
function utxo_check_expect_block_out_of_range(data) {
    if (data.error !== null) {
        if (data.error.code === -8) {
            return true;
        }
        if (data.error.code === -1) {
            return true;
        }
        else {
            // throw MccError(data);
            return false;
        }
    }
    else {
        return false;
    }
}
exports.utxo_check_expect_block_out_of_range = utxo_check_expect_block_out_of_range;
function utxo_ensure_data(data) {
    if (data.error !== null) {
        throw (0, utils_1.MccError)(data);
    }
}
exports.utxo_ensure_data = utxo_ensure_data;
//# sourceMappingURL=utxoUtils.js.map